/******************************************************************************
 DISCLAIMER

 THIS SOFTWARE PRODUCT ("SOFTWARE") IS PROPRIETARY TO ENOCEAN GMBH, OBERHACHING,
 GERMANY (THE "OWNER") AND IS PROTECTED BY COPYRIGHT AND INTERNATIONAL TREATIES OR
 PROTECTED AS TRADE SECRET OR AS OTHER INTELLECTUAL PROPERTY RIGHT. ALL RIGHTS, TITLE AND
 INTEREST IN AND TO THE SOFTWARE, INCLUDING ANY COPYRIGHT, TRADE SECRET OR ANY OTHER
 INTELLECTUAL PROPERTY EMBODIED IN THE SOFTWARE, AND ANY RIGHTS TO REPRODUCE,
 DISTRIBUTE, MODIFY, DISPLAY OR OTHERWISE USE THE SOFTWARE SHALL EXCLUSIVELY VEST IN THE
 OWNER. ANY UNAUTHORIZED REPRODUCTION, DISTRIBUTION, MODIFICATION, DISPLAY OR OTHER
 USE OF THE SOFTWARE WITHOUT THE EXPLICIT PERMISSION OF OWNER IS PROHIBITED AND WILL
 CONSTITUTE AN INFRINGEMENT OF THE OWNER'S RIGHT AND MAY BE SUBJECT TO CIVIL OR
 CRIMINAL SANCTION.

 THIS SOFTWARE IS PROVIDED BY THE OWNER "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN PARTICULAR, THE OWNER DOES NOT WARRANT
 THAT THE SOFTWARE SHALL BE ERROR FREE AND WORKS WITHOUT INTERRUPTION.

 IN NO EVENT SHALL THE OWNER BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/

/**
 * \file eoEEProfile.h
 * \brief Profile base class for Enocean Equipment Profiles

 * \author EnOcean GmBH
 */
#ifndef EO_EEPROFILE_H_
#define EO_EEPROFILE_H_

#include "eoProfile.h"

/**
 * \ingroup eepProfiles 
 * @{
 */
/**
 * \class eoEEProfile
 * \brief base class for EEProfile parsing
 * \details This class is the base class for all eoEEProfiles.
 * \example profile_examples.cpp
 */
class eoEEProfile: public eoProfile
{
public:
	/**
	 * Constructor
	 * @param size Default size of the expected message.
	 */
	eoEEProfile(uint16_t size = 4);
	virtual ~eoEEProfile();

	//virtual bool operator==(const eoProfile &othProfile) const;
	//virtual bool operator!=(const eoProfile &othProfile) const;
	/**
	 virtual eoReturn Create(eoMessage &m);
	 virtual eoReturn CreateTeachIN(eoMessage &m);

	 virtual eoReturn  Parse(const eoMessage &msg);

	 virtual eoReturn GetValue(CHANNEL_TYPE type, float &value);
	 virtual eoReturn GetValue(CHANNEL_TYPE type, float &value, uint8_t subFlag);
	 virtual eoReturn GetValue(CHANNEL_TYPE type, uint8_t &value);
	 virtual eoReturn GetValue(CHANNEL_TYPE type, uint8_t &value, uint8_t subFlag);

	 virtual eoReturn SetValue(CHANNEL_TYPE type,float value, uint8_t subFlag);
	 virtual eoReturn SetValue(CHANNEL_TYPE type,uint8_t value, uint8_t subFlag);

	 virtual eoReturn SetValue(CHANNEL_TYPE type, float value);
	 virtual eoReturn SetValue(CHANNEL_TYPE type, const uint8_t value);
	 **/
	virtual eoChannelInfo* GetChannel(CHANNEL_TYPE type, uint8_t subType);
	virtual eoChannelInfo* GetChannel(CHANNEL_TYPE type);
	virtual eoChannelInfo* GetChannel(uint8_t channelNumber);
	virtual eoReturn GetSubTypeChannel(CHANNEL_TYPE type, std::vector<eoEEPChannelInfo*>& vector);

	//virtual eoReturn SetType(uint8_t type);

	virtual uint8_t GetChannelCount() const;

protected:
	//! ChannelInfo list for direct Access to channels
	eoEEPChannelInfo* channel;
	//! Number of channels
	uint8_t channelCount;

};
/**
 * @}
 */

#endif /* EOEEPROFILE_H_ */
