/******************************************************************************
 DISCLAIMER

 THIS SOFTWARE PRODUCT ("SOFTWARE") IS PROPRIETARY TO ENOCEAN GMBH, OBERHACHING, 
 GERMANY (THE "OWNER") AND IS PROTECTED BY COPYRIGHT AND INTERNATIONAL TREATIES OR 
 PROTECTED AS TRADE SECRET OR AS OTHER INTELLECTUAL PROPERTY RIGHT. ALL RIGHTS, TITLE AND
 INTEREST IN AND TO THE SOFTWARE, INCLUDING ANY COPYRIGHT, TRADE SECRET OR ANY OTHER 
 INTELLECTUAL PROPERTY EMBODIED IN THE SOFTWARE, AND ANY RIGHTS TO REPRODUCE, 
 DISTRIBUTE, MODIFY, DISPLAY OR OTHERWISE USE THE SOFTWARE SHALL EXCLUSIVELY VEST IN THE
 OWNER. ANY UNAUTHORIZED REPRODUCTION, DISTRIBUTION, MODIFICATION, DISPLAY OR OTHER
 USE OF THE SOFTWARE WITHOUT THE EXPLICIT PERMISSION OF OWNER IS PROHIBITED AND WILL 
 CONSTITUTE AN INFRINGEMENT OF THE OWNER'S RIGHT AND MAY BE SUBJECT TO CIVIL OR 
 CRIMINAL SANCTION.

 THIS SOFTWARE IS PROVIDED BY THE OWNER "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN PARTICULAR, THE OWNER DOES NOT WARRANT 
 THAT THE SOFTWARE SHALL BE ERROR FREE AND WORKS WITHOUT INTERRUPTION.

 IN NO EVENT SHALL THE OWNER BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
 OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/

/**
 * \file eoManufacturer.h
 * \brief Manufacturer list
 * \author EnOcean GmbH
 */
#ifndef __EOMANUFACTURER_H
#define	__EOMANUFACTURER_H

#include "eoHalTypes.h"

/**\ingroup eoManufacturer
 * @{
 */

//! Manufacturer ID List
typedef enum
{
	MANUFACTURER_RESERVED = 0x000,
	PEHA = 0x001,
	THERMOKON = 0x002,
	SERVODAN = 0x003,
	ECHOFLEX_SOLUTIONS = 0x004,
	OMNIO_AG = 0x005,
	HARDMEIER_ELECTRONICS = 0x006,
	REGULVAR_INC = 0x007,
	AD_HOC_ELECTRONICS = 0x008,
	DISTECH_CONTROLS = 0x009,
	KIEBACK_AND_PETER = 0x00A,
	ENOCEAN_GMBH = 0x00B,
	PROBARE = 0x00C,
	ELTAKO = 0x00D,
	LEVITON = 0x00E,
	HONEYWELL = 0x00F,
	SPARTAN_PERIPHERAL_DEVICES = 0x010,
	SIEMENS = 0x011,
	T_MAC = 0x012,
	RELIABLE_CONTROLS_CORPORATION = 0x013,
	ELSNER_ELEKTRONIK_GMBH = 0x014,
	DIEHL_CONTROLS = 0x015,
	BSC_COMPUTER = 0x016,
	S_AND_S_REGELTECHNIK_GMBH = 0x017,
	MASCO_CORPORATION = 0x018,
	INTESIS_SOFTWARE_SL = 0x019,
	VIESSMANN = 0x01A,
	LUTUO_TECHNOLOGY = 0x01B,
	SCHNEIDER_ELECTRIC = 0x01C,
	SAUTER = 0x01D,
	BOOT_UP = 0x01E,
	OSRAM_SYLVANIA = 0x01F,
	UNOTECH = 0x020,
	DELTA_CONTROLS_INC = 0x21,
	UNITRONIC_AG = 0x022,
	NANOSENSE = 0x023,
	THE_S4_GROUP = 0x024,
	MSR_SOLUTIONS = 0x025,
	GE = 0x26,
	MAICO = 0x027,
	RUSKIN_COMPANY = 0x28,
	MAGNUM_ENERGY_SOLUTIONS = 0x29,
	KMC_CONTROLS = 0x02A,
	ECOLOGIX_CONTROLS = 0x02B,
	TRIO_2_SYS = 0x2C,
	AFRISO_EURO_INDEX = 0x02D,
	NEC_ACCESSTECHNICA_LTD = 0x030,
	ITEC_CORPORATION = 0x031,
	SIMICX_CO_LTD = 0x32,
	EUROTRONIC_TECHNOLOGY_GMBH = 0x34,
	ART_JAPAN_CO_LTD = 0x35,
	TIANSU_AUTOMATION_CONTROL_SYSTE_CO_LTD = 0x36,
	GRUPPO_GIORDANO_IDEA_SPA = 0x38,
	ALPHAEOS_AG = 0x39,
	TAG_TECHNOLOGIES = 0x3A,
	CLOUD_BUILDINGS_LTD = 0x3C,
	GIGA_CONCEPT = 0x3E,
	SENSORTEC = 0x3F,
	JAEGER_DIREKT = 0x40,
	AIR_SYSTEM_COMPONENTS_INC = 0x41,
	MULTI_USER_MANUFACTURER = 0x7FF
} MANUFACTURER_LIST;

#define MAX_NAME_LENGTH 42

/**
 * \class eoManufacturer
 *  \brief Manufacturer List
 *  \details The eoManufacturer class provides functionality to convert manufacturer ID to name.
 */
class eoManufacturer
{
private:

	const static char	list[][MAX_NAME_LENGTH];
	const static char	listDefault[2][MAX_NAME_LENGTH];

public:
	virtual ~eoManufacturer();

	/**
	 * Returns the name of manufacturer as zero terminated string
	 * @return Manufacturer name
	 */
	static const char* GetName(const uint16_t manufacturerID);
};
/**
 * @}
 */


#endif // __EOMANUFACTURER_H
