/******************************************************************************
 DISCLAIMER

 THIS SOFTWARE PRODUCT ("SOFTWARE") IS PROPRIETARY TO ENOCEAN GMBH, OBERHACHING, 
 GERMANY (THE "OWNER") AND IS PROTECTED BY COPYRIGHT AND INTERNATIONAL TREATIES OR 
 PROTECTED AS TRADE SECRET OR AS OTHER INTELLECTUAL PROPERTY RIGHT. ALL RIGHTS, TITLE AND
 INTEREST IN AND TO THE SOFTWARE, INCLUDING ANY COPYRIGHT, TRADE SECRET OR ANY OTHER 
 INTELLECTUAL PROPERTY EMBODIED IN THE SOFTWARE, AND ANY RIGHTS TO REPRODUCE, 
 DISTRIBUTE, MODIFY, DISPLAY OR OTHERWISE USE THE SOFTWARE SHALL EXCLUSIVELY VEST IN THE
 OWNER. ANY UNAUTHORIZED REPRODUCTION, DISTRIBUTION, MODIFICATION, DISPLAY OR OTHER
 USE OF THE SOFTWARE WITHOUT THE EXPLICIT PERMISSION OF OWNER IS PROHIBITED AND WILL 
 CONSTITUTE AN INFRINGEMENT OF THE OWNER'S RIGHT AND MAY BE SUBJECT TO CIVIL OR 
 CRIMINAL SANCTION.

 THIS SOFTWARE IS PROVIDED BY THE OWNER "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN PARTICULAR, THE OWNER DOES NOT WARRANT 
 THAT THE SOFTWARE SHALL BE ERROR FREE AND WORKS WITHOUT INTERRUPTION.

 IN NO EVENT SHALL THE OWNER BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
 OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/

#include "eoTimer.h"
#include <time.h>
#ifdef WIN32
#include <windows.h>
#elif defined POSIX
#include <errno.h>
#endif
eoTimer::eoTimer()
{

}

eoTimer::~eoTimer()
{

}

uint32_t eoTimer::GetTickCount()
{
#ifdef WIN32
	return ::GetTickCount();
#elif defined POSIX
	struct timespec ts;
	if (clock_gettime(CLOCK_MONOTONIC, &ts) != 0)
	{
		//error
		return 0;
	}
	//gettime_count is buggy somehow, see boost for an own implementation
	return (ts.tv_nsec) / 1000 / 1000 + ts.tv_sec * 1000;
#else
	return 0;
#endif
}

void eoTimer::Sleep(uint32_t ms)
{
#ifdef WIN32
	//Not buffer overflow save...
	/*uint32_t currentTicks=GetTickCount();
	uint32_t expectedTicks=currentTicks+ms;
	if(expectedTicks>currentTicks)
	{
		while(expectedTicks>currentTicks)
		{
			currentTicks=GetTickCount();
		}
	}
	else
	{
		while(expectedTicks<currentTicks)
		{
			currentTicks=GetTickCount();
		}
	}*/

	::Sleep(ms);
#elif defined POSIX
	struct timespec ts;
	struct timespec rs;
	//1Millisecond = 1000 * 1000 ns;
	ts.tv_nsec=ms*1000*1000;
	while(nanosleep(&ts,&rs)!=0 && errno==EINTR)
	{
		ts=rs;
	}
#endif

}
